from .abc import Definition, NewValues
from damn.value import Value, DecimalValue
import typing
import re
import random


class InPairDefinition(Definition):
    pattern = re.compile(r"\((.+)\) in \[(.+)]")
    pair_pattern = re.compile(r"\(([^()]+)\)")

    def __init__(
        self, names: typing.List[str], pairs: typing.List[typing.Tuple[Value, ...]]
    ):
        self.names = names
        self.pairs = pairs

    @classmethod
    def parse(cls, s: str) -> typing.Optional["InPairDefinition"]:
        m = re.match(cls.pattern, s)
        if not m:
            return None
        m = m.groups()
        names = list(map(lambda x: x.strip(" "), m[0].split(",")))
        pairs = []
        for pair in re.finditer(cls.pair_pattern, m[1]):
            pair = pair.group(1)
            mapping = map(lambda x: x.strip(" "), pair.split(","))
            values = map(lambda x: DecimalValue.from_str(x), mapping)
            pairs.append(tuple(values))
        return cls(names, pairs)

    def generate(self) -> NewValues:
        pair = random.choice(self.pairs)
        return dict(zip(self.names, pair))



import typing
from .definition import (
    Definition,
    InPairDefinition,
    InSingleDefinition,
    DomainDefinition,
)
from .value import Value, DecimalValue
from .lib import wolfram_calculate
import itertools

definition_types = [InPairDefinition, InSingleDefinition, DomainDefinition]


class DefinitionSet:
    def __init__(self, def_set: typing.Set[Definition]):
        self.def_set = def_set

    @classmethod
    def from_expressions(cls, expressions: typing.List[str]) -> "DefinitionSet":
        def_set = set()
        for expr in expressions:
            definition: typing.Optional["Definition"] = None
            for deft in definition_types:
                if d := deft.parse(expr):
                    definition = d
            if not definition:
                raise SyntaxError("Unable to parse expression `{}`".format(expr))
            def_set.add(definition)
        return cls(def_set)

    def get_data(self) -> typing.Dict[str, Value]:
        data = {}
        for definition in self.def_set:
            data.update(definition.generate())
        return data



from .abc import Value
from damn.utils import braced


class RationalValue(Value):
    def __init__(self, nom: Value, denom: Value):
        self.nom = nom
        self.denom = denom

    def get_common_representation(self) -> str:
        return (
            self.nom.get_common_representation()
            + "/"
            + self.denom.get_common_representation()
        )

    def get_formula_representation(self) -> str:
        return braced(
            self.nom.get_formula_representation()
            + "/"
            + self.denom.get_formula_representation()
        )

    def cmp_to_user_input(self, s: str) -> bool:
        s = s.replace(" ", "").replace(",", ".")
        return (
            s == self.get_common_representation()
            or s == self.get_formula_representation()
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['damn', 'damn.definition', 'damn.lib', 'damn.value']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.3,<4.0.0']

setup_kwargs = {
    'name': 'damnlang',
    'version': '0.1.0',
    'description': 'Damn',
    'long_description': '# damn\n\n**damn** (desperately agonizing math noobs) is a language to help you write knowledge assessment test generators.\n\nLanguage will help you in variables constraints declaration and generate unique pairs of data. Then, the generated pair of variables can be evaluated with the formula (via wolframalpha integration).\n\nSyntax:\n\n```damn\n1 > x > 10\ny in [1, 2, 3]\n(a, b) in [(4, 5), (1, 10)]\n```\n\n## Example\n\nConstraints:\n\n```damn\n10 > a > 20\n(v1, v2) in [(1, 2), (4, 8)]\n```\n\nFormula:\n\n```damn\na = (v2 - v1)/t\n```\n\nMaking a test:\n\n```python\nfrom damn import DefinitionSet, Calculator\nimport asyncio\n\ncalculator = Calculator(["DEMO"])\n\nasync def main():\n    def_set = DefinitionSet.from_expressions([\n        "10 > a > 20", \n        "(v1, v2) in [(1, 2), (4, 8)]"\n    ])\n    data = def_set.get_data()\n    print(\n        "What is delta t if " + \n        ", ".join(\n            (f"{k} = {v.get_common_representation()}" for k, v in data.items())\n        )\n    )\n    result = await calculator.calculate(data, "a = (v2 - v1)/t")\n    answer = float(input("your answer > "))\n    if answer == result.v:\n        print("great!!")\n        return\n    print(f"BAD student. the right answer is {result.v}")\n\nasyncio.run(main())\n```',
    'author': 'timoniq',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from abc import ABC, abstractmethod
from damn.value.abc import Value
from damn.error import ParseError
import typing


NewValues = typing.Dict[str, Value]


class Definition(ABC):
    @classmethod
    @abstractmethod
    def parse(cls, s: str) -> typing.Optional["Definition"]:
        pass

    @abstractmethod
    def generate(self) -> NewValues:
        pass

    @classmethod
    def get_error(cls, err: str = ""):
        return ParseError(cls, err)
